% Add sequential measurements to the basis pursuit problem

clear

% source length
n = 256;
% codeword length
m = 100;
% number of non-zero elements
T = round(m/2);

% measurement matrix
A_mat = randn(m,n);
b_new = randn(1,n);

% sparse signal
q = randperm(n-1);
x = zeros(n,1);
x(q(1:T)) = randsrc(T,1);
y = A_mat*x;

%%
% Solve BP to get the initial solution with m measurements
cvx_begin
    cvx_precision best
    variable x_old(n);
    dual variable lam_old;
    minimize( norm( x_old, 1 ) )
    subject to
    lam_old: A_mat * x_old == y;
cvx_end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%
% DynamicSeq-BP Homotopy %
%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add one new measurement to the system and update the solution.

% New measurement
w = b_new*x;

gamma = find(abs(x_old)>1e-5);
if length(gamma)~=m
    stp = 1;
    why
end

% Initialization
del_lam = -A_mat(:,gamma)'\b_new(gamma)';
gamma_c = setdiff(1:n,gamma);
ak = A_mat'*lam_old;
ak(gamma) = sign(ak(gamma));
bk = A_mat'*del_lam + b_new';

theta_p1 = (1-ak(gamma_c))./bk(gamma_c);
theta_n1 = (-1-ak(gamma_c))./bk(gamma_c);

[theta1 ind_1] = min(abs(theta_p1));
[theta2 ind_2] = min(abs(theta_n1));
if theta1>theta2
    theta_in = theta_n1(ind_2);
    itheta_in = gamma_c(ind_2);
else
    theta_in = theta_p1(ind_1);
    itheta_in = gamma_c(ind_1);
end

lamk_1 = lam_old+theta_in*del_lam;
nuk_1 = theta_in;
gamma_new = [gamma; itheta_in];
xk_1 = x_old;
xk_1(itheta_in) = 0;
epsilon_new = 0;
G =  [A_mat; b_new];
ak_old = ak+theta_in*bk;
ak_old(gamma_new) = sign(ak_old(gamma_new));

done = 0;
iter = 0;
% Homotopy primal and dual iterations
while ~done
    iter = iter+1;
    epsilon = epsilon_new;
    x_k = xk_1;
    
    rv = [zeros(m,1); -b_new*x_old+w];
    delx_vec = zeros(n,1);
    
    % NEED FAST UPDATE HERE
    del_x = G(:,gamma_new)\rv;
    delx_vec(gamma_new) = del_x;
    theta_X = -(x_k(gamma_new)./del_x);
    theta_outp = theta_X(theta_X>2*eps);
    theta_out = min(theta_outp);
    itheta_out = gamma_new(find(theta_X==theta_out));
    % There will be a sign ambiguity for the first iteration! Live with it.
    % The sign of dual variable given by CVX is opposite to what I have used
    % in the formulation!
    if iter == 1
        if sign(delx_vec(itheta_in)) ~= sign(ak_old(itheta_in))
            theta_out = 0;
            itheta_out = itheta_in;
            why
        end
    end
    xk_1 = x_k + theta_out*delx_vec;
    xk_1(itheta_out) = 0;
    epsilon_new = epsilon+theta_out;

    if epsilon_new > 1
        theta_out_end = 1-epsilon;
        xk_1 = x_k + theta_out_end*delx_vec;
        break;
    end

    lam_k = lamk_1;
    nu_k = nuk_1;

    gamma_out = itheta_out;
    gamma_dual = setdiff(gamma_new,gamma_out);
    
    % NEED FAST UPDATE HERE
    del_lam = -A_mat(:,gamma_dual)'\b_new(gamma_dual)';
    gamma_c = setdiff(1:n,gamma_dual);
    % I think gamma_c should include the itheta_out as well, because what
    % if the signal changes polarity?? So put a check that if theta_in
    % becomes greater than 2 it indicates that outgoing element has entered
    % back with opposite polarity.
    ak = ak_old;
    bk = A_mat'*del_lam + b_new';
    sign_out_theta = -sign(bk(gamma_out))*sign(ak(gamma_out));
    
    if sign_out_theta == -1
        bk = -bk;
    end
    
    constr1 = (1-ak(gamma_c))./bk(gamma_c);
    constr1_pos = constr1(constr1>2*eps);
    delta1 = min(constr1_pos);
    idelta1 = gamma_c(find(constr1==delta1));

    constr2 = (-1-ak(gamma_c))./bk(gamma_c);
    constr2_pos = constr2(constr2>2*eps);
    delta2 = min(constr2_pos);
    idelta2 = gamma_c(find(constr2==delta2));

    if delta1>delta2
        theta_in = delta2;
        itheta_in = idelta2;
    else
        theta_in = delta1;
        itheta_in = idelta1;
    end
%         constr1 = (1-ak(gamma_c))./bk(gamma_c);
%         constr1_pos = constr1(constr1<-2*eps);
%         delta1 = max(constr1_pos);
%         idelta1 = gamma_c(find(constr1==delta1));
%         
%         constr2 = (-1-ak(gamma_c))./bk(gamma_c);
%         constr2_pos = constr2(constr2<-2*eps);
%         delta2 = max(constr2_pos);
%         idelta2 = gamma_c(find(constr2==delta2));
%         
%         if delta1>delta2
%             theta_in = delta2;
%             itheta_in = idelta2;
%         else
%             theta_in = delta1;
%             itheta_in = idelta1;
%         end

%     cvx_begin
%         cvx_precision best
%         cvx_quiet(1)
%         variable xt_eq(n);
%         dual variables lam nu;
%         minimize( norm( xt_eq, 1 ) )
%         subject to
%         lam: A_mat*xt_eq == y;
%         nu: b_new*xt_eq == (1-epsilon_new*1.01)*b_new*x_old+epsilon_new*1.01*w;
%         %lam: A_mat*xt_eq == (1-wt)*y+wt*y_new;
%     cvx_end
    
    xk_1(itheta_in) = 0;
    lamk_1 = lam_k+theta_in*del_lam*sign_out_theta;
    nuk_1 = nu_k+theta_in*sign_out_theta;

    gamma_new = [gamma_dual; itheta_in];
    ak_old = ak+theta_in*bk;
    ak_old(gamma_new) = sign(ak_old(gamma_new));
    % norm(xk_1-xt_eq)
    [iter epsilon_new itheta_in itheta_out  theta_in theta_out]
end
% Homotopy output
x_h = xk_1;

%%
% Check the solution with new set of measurements.
cvx_begin
    cvx_precision high
    variable x_new(n);
    minimize( norm( x_new, 1 ) )
    subject to
    A_mat * x_new == y;
    b_new * x_new == w;
cvx_end

[iter norm(x_new -x_h)]
