% Run dynamic updating once so that we have the initial support for the
% homotopy and then make some changes in the signal and run dynamic BP again
clear

% % load fixed random states
load RandomStates
rand('state', s_rand);
randn('state', s_randn);

% source length
N = 256;
% codeword length
M = 128;
% number of perturbations
T = round(M/5);

A = randn(N,N);
[Q R] = qr(A);
A_mat = Q(1:M,:);

% channel: perturb T randomly chosen entries
q = randperm(N-1);
x = zeros(N,1);
x(q(1:T)) = randsrc(T,1);

y = A_mat*x;

cvx_begin
    cvx_precision best
    variable x_old(N);
    dual variable lam_old;
    minimize( norm( x_old, 1 ) )
    subject to
    lam_old: y==A_mat * x_old ;
cvx_end

x0 = A_mat \y;

%%
x2 = zeros(N,1);
p = randperm(N);
x2(q(1:T)) = x(q(1:T))+.1*randn(T,1);
T_new = round(T/5);
x2(q(T+1:T+T_new)) = randn(T_new,1);
y_new = A_mat*x2;

gamma_test_new = find(abs(abs(A_mat'*lam_old)-1)<1e-8);
gamma_test_new = find(abs(x_old)>1e-8);
lam_new = lam_old;
gamma_0 = gamma_test_new;
lam_0 = lam_new;
x_0 = zeros(N,1);
x_0(gamma_test_new) = x_old(gamma_test_new);
ak_old = A_mat'*lam_0;
ak_old(gamma_test_new) = sign(gamma_test_new);
all_done = 0;

for iter = 1:M-length(gamma_test_new)
    lam_old = lam_new;
    gamma_test_old = gamma_test_new;
    gamma_xc = setdiff([1:N],gamma_test_old);
    Atg = A_mat(:,gamma_test_old)';
    P = eye(M)-Atg'*inv(Atg*Atg')*Atg;
    del_lam = -P*(y+y_new);
    if norm(del_lam)<1e-10 
        stp =1;
        % In this case solve a least squares problem because now the
        % problems are separable.
        x_combined = zeros(N,1);
        x_combined(gamma_test_old) = A_mat(:,gamma_test_old)\(y+y_new);
        all_done = 1;
        break;
    end
    ak = ak_old;
    bk = A_mat'*del_lam;
    b_constr1 = (1-ak(gamma_xc))./bk(gamma_xc);
    b_constr2 = (1+ak(gamma_xc))./-bk(gamma_xc);
    itheta_1 = find(b_constr1>2*eps);
    itheta_2 = find(b_constr2>2*eps);
    theta1 = min(b_constr1(itheta_1));
    theta2 = min(b_constr2(itheta_2));
    if theta1>theta2
        theta = theta2;
        itheta = gamma_xc(b_constr2==theta2);
    else
        theta = theta1;
        itheta = gamma_xc(b_constr1==theta1);
    end
    lam_new = lam_old+theta*del_lam;
    gamma_test_new = [gamma_test_old; itheta];
    if length(gamma_test_new)==M
        stp =1;
        break;
    end
    [itheta A_mat(:,itheta)'*lam_new max(abs(A_mat(:,q(1:T))'*lam_new))]
    ak_old = ak+theta*bk;
    ak_old(gamma_test_new) = sign(ak_old(gamma_test_new));
    
    if abs(max(abs(A_mat'*lam_new))-1)>1e-8
        stp =1;
    end
end

e0 = 0;
gamma_temp = setdiff(gamma_test_new,gamma_0);
xp_h = x_old;
xp_h(abs(xp_h)<1e-10) = 0;
xp_h(gamma_temp) = -sign(A_mat(:,gamma_temp)'*lam_new)*5*eps;

gamma_xh = gamma_test_new;
lambda_h = lam_new;
ak_old = A_mat'*lambda_h;
ak_old(gamma_xh) = sign(ak_old(gamma_xh));

done = 0;
hom_iter = 0;
while ~done & ~all_done
    if nnz(sign(xp_h(gamma_xh))+sign(ak_old(gamma_xh))) > 1
        stp =1;
    end

    hom_iter = hom_iter+1;
    delx = -(A_mat(:,gamma_xh))\(y-y_new);
    delx_vec = zeros(N,1);
    delx_vec(gamma_xh) = delx;
    b_constr3 = (-xp_h(gamma_xh)./delx_vec(gamma_xh));
    itheta_3 = find(b_constr3>0);
    theta_out = min(b_constr3(itheta_3));
    itheta_out = gamma_xh(b_constr3==theta_out);
    
    if e0+theta_out >=1
        theta_end = (1-e0);
        xp_h = xp_h + theta_end*delx_vec;
        break;
    end
    if norm(xp_h-x2) < 1e-4
        stp =1;
        break;
    end
    xp_old = xp_h;
    xp_h = xp_h + theta_out*delx_vec;
    xp_h(itheta_out) = 0;
    epsilon = e0+theta_out;
    
    [theta_out epsilon]
    if theta_out < 1e-6
        stp = 1;
    end
%     wt = epsilon
%     cvx_begin
%         cvx_precision best
%         cvx_quiet(1)
%         variable xt_eq(N);
%         dual variables lam nu;
%         minimize( norm( xt_eq, 1 ) )
%         subject to
%         lam: (1-wt)*y+wt*y_new==A_mat*xt_eq;
%     cvx_end
    
    gamma_xn = setdiff(gamma_xh,itheta_out);
    gamma_xc = setdiff([1:N],gamma_xn);
    
    ak = ak_old;
    % z_x = sign(xp_old(itheta_out));
    z_x = -sign(ak(itheta_out));
    del_lam = (A_mat(:,[gamma_xn; itheta_out])')\([zeros(M-1,1); z_x]);
    bk = A_mat'*del_lam;
    b_constr1 = (1-ak(gamma_xc))./bk(gamma_xc);
    b_constr2 = (1+ak(gamma_xc))./-bk(gamma_xc);
    itheta_1 = find(b_constr1>2*eps);
    itheta_2 = find(b_constr2>2*eps);
    theta1 = min(b_constr1(itheta_1));
    theta2 = min(b_constr2(itheta_2));
    if theta1>theta2
        theta_in = theta2;
        itheta_in = gamma_xc(find(b_constr2==theta2));
    else
        theta_in = theta1;
        itheta_in = gamma_xc(find(b_constr1==theta1));
    end
    lambda_old = lambda_h;
    lambda_h = lambda_h + theta_in*del_lam;
    gamma_xh = [gamma_xn; itheta_in];
    figure(1); plot(A_mat'*lambda_h);
    
    e0 = epsilon;
    ak_old = ak+theta_in*bk;
    ak_orig = ak_old;
    if abs(max(abs(ak_old))-1)>1e-10
        stp =1;
    end
    ak_old(gamma_xh) = sign(ak_old(gamma_xh));
%     figure(1); clf; plot(A_mat'*lambda_h); hold on; plot(A_mat'*lam,'ro'); shg
end

%%
lam_old = lambda_h;
x_old = xp_h;
y = y_new;
% x2 = zeros(N,1);
p = randperm(N);
% T = round(M/3);
% x2(q(1:T)) = x2(q(1:T))+.1*randn(T,1);
all_done = 0;
T_new = round(T/5);
x2(q(T+1:T+T_new)) = randn(T_new,1);
y_new = A_mat*x2;

lam_new = lam_old;
gamma_test_new = gamma_xh;
gamma_0 = gamma_test_new;
lam_0 = lam_new;
x_0 = x_old;
ak_old = A_mat'*lam_0;
ak_old(gamma_test_new) = sign(gamma_test_new);

e0 = 0;
xp_h = x_old;

gamma_xh = gamma_test_new;
lambda_h = lam_new;
ak_old = A_mat'*lambda_h;
ak_old(gamma_xh) = sign(ak_old(gamma_xh));

done = 0;
hom_iter = 0;
while ~done && ~all_done
    if nnz(sign(xp_h(gamma_xh))+sign(ak_old(gamma_xh))) > 1
        stp =1;
    end

    hom_iter = hom_iter+1;
    delx = -(A_mat(:,gamma_xh))\(y-y_new);
    delx_vec = zeros(N,1);
    delx_vec(gamma_xh) = delx;
    b_constr3 = (-xp_h(gamma_xh)./delx_vec(gamma_xh));
    itheta_3 = find(b_constr3>0);
    theta_out = min(b_constr3(itheta_3));
    itheta_out = gamma_xh(b_constr3==theta_out);
    
    if e0+theta_out >=1
        theta_end = (1-e0);
        xp_h = xp_h + theta_end*delx_vec;
        break;
    end
    if norm(xp_h-x2) < 1e-2
        stp =1;
        break;
    end
    xp_old = xp_h;
    xp_h = xp_h + theta_out*delx_vec;
    xp_h(itheta_out) = 0;
    epsilon = e0+theta_out;
    
    [theta_out epsilon]
    if theta_out < 1e-6
        stp = 1;
    end
%     wt = epsilon
%     cvx_begin
%         cvx_precision best
%         cvx_quiet(1)
%         variable xt_eq(N);
%         dual variables lam nu;
%         minimize( norm( xt_eq, 1 ) )
%         subject to
%         lam: (1-wt)*y+wt*y_new==A_mat*xt_eq;
%     cvx_end
    
    gamma_xn = setdiff(gamma_xh,itheta_out);
    gamma_xc = setdiff([1:N],gamma_xn);
    
    ak = ak_old;
    % z_x = sign(xp_old(itheta_out));
    z_x = -sign(ak(itheta_out));
    del_lam = (A_mat(:,[gamma_xn; itheta_out])')\([zeros(M-1,1); z_x]);
    bk = A_mat'*del_lam;
    b_constr1 = (1-ak(gamma_xc))./bk(gamma_xc);
    b_constr2 = (1+ak(gamma_xc))./-bk(gamma_xc);
    itheta_1 = find(b_constr1>2*eps);
    itheta_2 = find(b_constr2>2*eps);
    theta1 = min(b_constr1(itheta_1));
    theta2 = min(b_constr2(itheta_2));
    if theta1>theta2
        theta_in = theta2;
        itheta_in = gamma_xc(b_constr2==theta2);
    else
        theta_in = theta1;
        itheta_in = gamma_xc(b_constr1==theta1);
    end
    lambda_old = lambda_h;
    lambda_h = lambda_h + theta_in*del_lam;
    gamma_xh = [gamma_xn; itheta_in];
    figure(1); plot(A_mat'*lambda_h);
    
    e0 = epsilon;
    ak_old = ak+theta_in*bk;
    ak_orig = ak_old;
    if abs(max(abs(ak_old))-1)>1e-10
        stp =1;
    end
    ak_old(gamma_xh) = sign(ak_old(gamma_xh));
%     figure(1); clf; plot(A_mat'*lambda_h); hold on; plot(A_mat'*lam,'ro'); shg
end

%%

if all_done
    x_rec = x_combined - x_old;
else
    x_rec = xp_h;
end
cvx_begin
    cvx_precision best
    cvx_quiet(1)
    variable xt_eq(N);
    dual variables lam;
    minimize( norm( xt_eq, 1 ) )
    subject to
    lam: y_new==A_mat*xt_eq;
cvx_end
figure(101); plot([x_rec x2]);
figure(102); plot([x_rec xt_eq]);
