clear
% load fixed random states
% load RandomStates
% rand('state', s_rand);
% randn('state', s_randn);

% source length
N = 256;
% codeword length
M = 128;
% number of perturbations
T = round(M/5);

A = randn(N,N);
[Q R] = qr(A);
A_mat = Q(1:M,:);
m_u = 1;
b_new = Q(M+1:M+m_u,:);
Q_old = b_new';
Q_mat = Q(M+1:end,:)';

% channel: perturb T randomly chosen entries
q = randperm(N-1);
x = zeros(N,1);
x(q(1:T)) = randsrc(T,1);

y = A_mat*x;
w = b_new*x;

cvx_begin
    cvx_precision high
    variable x_new(N);
    minimize( norm( x_new, 1 ) )
    subject to
    y==A_mat * x_new;
    w==b_new * x_new;
cvx_end

cvx_begin
    cvx_precision low
    variable x_old(N);
    dual variable lam_old;
    minimize( norm( x_old, 1 ) )
    subject to
    lam_old: y==A_mat * x_old ;
cvx_end

x0 = A_mat \y;

% cvx_begin
%     cvx_precision high
%     variables h_old(N-M)
%     minimize( norm( [Q_mat * h_old + x0], 1 ) )
% cvx_end
% 
% x1 = [A_mat; b_new]\[y;w];
% cvx_begin
%     cvx_precision high
%     variables h_new(N-M-1)
%     dual variable nu2;
%     minimize( norm( [Q_mat(:,2:end) * h_new + x1], 1 ) )
% cvx_end
%%
iter_len = 10;
wt_table = linspace(0.0,1,iter_len);

marker = ['b';'r';'k';'g';'m'];
m_ind = 1;

Qtilde = Q(M+1,:)'*[1 zeros(1,size(Q_mat,2)-1)];
dy = randn(M,1)*.1;
x2 = zeros(N,1);
p = randperm(N);
x2(q(T+1:T+T)) = randn(T,1);
y_new = A_mat*x2;
% figure(101); clf; hold on; plot(x_new,'r.'); plot(x_old, 'k.'); shg
figure(102); clf; hold on;
% figure(103); clf; hold on;
figure(101); clf; hold on;
xt_eq = x_old;
gamma_test_old = find(abs(x_old)>1e-6);
gamma_old = gamma_test_old;

for iter = 1:iter_len
    wt = wt_table(iter);

    cvx_begin
        cvx_precision best
        cvx_quiet(1)
        variable xt_eq(N);
        dual variables lam nu;
        minimize( norm( xt_eq, 1 ) )
        subject to
        %lam: A_mat*xt_eq == y;
        %nu: b_new*xt_eq == (1-wt)*b_new*x_old+wt*w;
        lam: (1-wt)*y+wt*y_new==A_mat*xt_eq;
    cvx_end
    gamma_test_new = find(abs(xt_eq)>1e-6);
    %if ~isempty(setxor(gamma_test_new,gamma_test_old))
        stp = 1;
        figure(102); plot([xt_eq],[marker(m_ind),'-']);
        %     figure(103); plot([lam],[marker(m_ind),'-']); shg
%         figure(101); plot(A_mat'*lam+b_new'*nu,[marker(m_ind),'-']); shg
        figure(101); plot(A_mat'*lam,[marker(m_ind),'-']);  drawnow
        
        m_ind = mod(m_ind+1,5)+1;
        diff_gamma = setxor(gamma_test_new,gamma_test_old);
        gamma_test_old = gamma_test_new;
        xt_old = xt_eq;
    %end
%     cvx_begin
%         cvx_precision best
%         cvx_quiet(1)
%         variables xt(N) hp(N-M);
%         dual variables lam1 ;
%         minimize( norm( xt, 1 ) )
%         subject to 
%         lam1: xt == [(Q_mat -wt*Qtilde) * hp + (1-wt)*x0+wt*x1];
%     cvx_end
    
%     cvx_begin
%         cvx_precision best
%         cvx_quiet(1)
%         variables xt(N) hp(N-M);
%         dual variables lam1 nu1;
%         minimize( norm( xt, 1 ) )
%         subject to 
%         lam1: xt == [Q_mat * hp + (1-wt)*x_old+wt*x1];
%         nu1: hp(1) == (1-wt)*h_old(1);
%     cvx_end



    [wt nu]
end

% G = [A; wt*b];
% Q = eye(size(G,1)-size(G,2),size(G,1))*(eye(size(G,1))-G*inv(G'*G)*G');
% y2 = [y; wt*w];
% cvx_begin
%     variable e(M+1);
%     dual variable lam;
%     minimize( norm( [e(1:end-1); e(end)], 1 ) )
%     subject to
%     lam: Q * e == -Q * y2;
% cvx_end
% 
% G = [A; b];
% Q = eye(size(G,1)-size(G,2),size(G,1))*(eye(size(G,1))-G*inv(G'*G)*G');
% y2 = [y; w];
% cvx_begin
%     variable e2(M+1);
%     dual variable lam;
%     minimize( norm( [e2(1:end-1); wt*e2(end)], 1 ) )
%     subject to
%     lam: Q * e2 == -Q * y2;
% cvx_end
% figure(1); plot([e(1:end-1)-e2(1:end-1)])
