% Test file for LASSO (BPDN) update with multiple measurements
% minimize ||x||_1 + 1/2 ||Ax-y||_2^2 + ||Bx-w||_2^2
% using homotopy update method
%
% [xp_h, gamma_xh, iter, th] = DynamicSeq_BPDN_multi_function(A, B, GtGgx, iGtGgx, y, w, xp, gamma_x, pk, tau, maxiter)

% A: mxn matrix
% B: new rows in measurement matrix
% GtGgx: GtG submatrix at row and column indices at gamma_x. G = [A;B]
% iGtGgx: inv(GtGgx)
% y: old set of measurements
% w: new measurements
% xp: old solution
% gamma_x: old support
% pk: old constraints
% tau: threshold parameter.
% In simulation U = (A'A+B'B) works better

% BPDN solution update with dynamic measurements
% Author: Salman Asif, Georgia Tech
% Email: sasif@gatech.edu
% Created: October 2010
%
%-------------------------------------------+
% Copyright (c) 2010.  Muhammad Salman Asif
%-------------------------------------------+


function [xp_h, gamma_xh, iter, th] = DynamicSeq_BPDN_multi_function(A, B, AtAgx, iAtAgx, y, w, xp, gamma_x, pk, tau, maxiter, constraint_plots)

t0 = cputime;
N = length(xp);

xp_h = xp;
gamma_xh = gamma_x;
itheta = [];

wd = B*xp;

epsilon = 0;
e0 = 0;

% Gram matrix update
% GtGgx = A(:,gamma_xh)'*A(:,gamma_xh)+B(:,gamma_xh)'*B(:,gamma_xh);
pk_old = pk;

done = 0;
iter = 0;
itheta_stack = [];

while iter < maxiter
    iter = iter +1 ;
    
    d = B(:,gamma_xh)'*(w-wd);
    %%% NEED UPDATE
    % iAtAgx = inv(A(:,gamma_xh)'*A(:,gamma_xh)+B(:,gamma_xh)'*B(:,gamma_xh));
    iAtAgx = inv(A(:,gamma_xh)'*A(:,gamma_xh)+B(:,gamma_xh)'*B(:,gamma_xh));
    delx = iAtAgx*d;
    
    pk = pk_old;
    dk = A'*(A(:,gamma_xh)*delx)+B'*(B(:,gamma_xh)*delx-(w-wd));
    delx_vec = zeros(N,1);
    delx_vec(gamma_xh) = delx;
    
    temp_gamma = zeros(N,1);
    temp_gamma(gamma_xh) = gamma_xh;
    gamma_xc = find([1:N]' ~= temp_gamma);
    % gamma_xc = setdiff([1:N],[gamma_xh]);
    
    b_constr1 = (tau-pk(gamma_xc))./dk(gamma_xc);
    b_constr2 = (tau+pk(gamma_xc))./-dk(gamma_xc);
    b_constr3 = (-xp_h(gamma_xh)./delx_vec(gamma_xh));
    itheta_1 = find(b_constr1>2*0);
    itheta_2 = find(b_constr2>2*0);
    itheta_3 = find(b_constr3>2*0);
    theta1 = min(b_constr1(itheta_1));
    theta2 = min(b_constr2(itheta_2));
    theta3 = min(b_constr3(itheta_3));
    if theta1>theta2
        theta = theta2;
        itheta = gamma_xc(find(b_constr2==theta2));
        flag = 1;
    else
        theta = theta1;
        itheta = gamma_xc(find(b_constr1==theta1));
        flag = 1;
    end
    if theta3 < theta
        theta = theta3;
        itheta = gamma_xh(find(b_constr3==theta3));
        flag = 0;
    end
    %     if flag == 0
    %         [epsilon+theta theta itheta flag xp2_h(itheta) delx_vec(itheta)]
    %     else
    %         [epsilon+theta theta itheta flag pk(itheta) dk(itheta)]
    %     end
    
    epsilon = (theta+e0);
    if epsilon < 0
        epsilon= inf;
    end
    gamma_xh_old = gamma_xh;
    xp_old = xp_h;
    xp_h = xp_h+theta*delx_vec;
    if flag == 1
        %         AtgxAnx = A(:,gamma_xh)'*A(:,itheta);
        %         AtAgx_mod = [AtAgx AtgxAnx; AtgxAnx' A(:,itheta)'*A(:,itheta)];
        %
        %         iAtAgx = update_inverse(AtAgx_mod, iAtAgx,1);
        %         AtAgx = AtAgx_mod;
        
        gamma_xh = [gamma_xh; itheta];
        itheta_stack = [itheta_stack; itheta];
    else
        outx_index = find(gamma_xh==itheta);
        gamma_xh(outx_index) = gamma_xh(end);
        gamma_xh(end) = itheta;
        gamma_xh = gamma_xh(1:end-1);
        
        itheta_stack = [itheta_stack; -itheta];
        
        %         rowi = outx_index; % ith row of A is swapped with last row (out_x)
        %         colj = outx_index; % jth column of A is swapped with last column (out_x)
        %         AtAgx_ij = AtAgx;
        %         temp_row = AtAgx_ij(rowi,:);
        %         AtAgx_ij(rowi,:) = AtAgx_ij(end,:);
        %         AtAgx_ij(end,:) = temp_row;
        %         temp_col = AtAgx_ij(:,colj);
        %         AtAgx_ij(:,colj) = AtAgx_ij(:,end);
        %         AtAgx_ij(:,end) = temp_col;
        %         iAtAgx_ij = iAtAgx;
        %         temp_row = iAtAgx_ij(colj,:);
        %         iAtAgx_ij(colj,:) = iAtAgx_ij(end,:);
        %         iAtAgx_ij(end,:) = temp_row;
        %         temp_col = iAtAgx_ij(:,rowi);
        %         iAtAgx_ij(:,rowi) = iAtAgx_ij(:,end);
        %         iAtAgx_ij(:,end) = temp_col;
        %
        %         AtAgx = AtAgx_ij(1:end-1,1:end-1);
        %         iAtAgx = update_inverse(AtAgx_ij, iAtAgx_ij,2);
    end
    
    %[epsilon theta itheta flag]
    %     itheta_stack = [itheta_stack; itheta];
    %     flag_stack = [flag_stack; flag];
    
    xp_h(itheta) = 0;
    
    if epsilon >=1
        theta_end = (1-e0);
        xp_h = xp_old + theta_end*delx_vec;
        gamma_xh = gamma_xh_old;
        epsilon = 1;
        %disp('done!');
        break;
    end
    
    e0 = (theta+e0);
    
    pk_old = pk+theta*dk;
    pk_old([gamma_xh; itheta]) = sign(pk_old([gamma_xh; itheta]))*tau;
    
    if constraint_plots
        figure(2);
        subplot(2,1,1);
        hold off;
        plot(pk,'.r','MarkerSize',14);
        hold on;
        plot(pk_old, 'LineWidth',1);
        
        if flag == 1
            plot(itheta, pk_old(itheta),'or','MarkerSize',18, 'LineWidth',2);
            text(itheta,pk_old(itheta)*1.2, ['Incoming \gamma = ',num2str(itheta)],'FontSize',14);
        else
            plot(itheta, pk_old(itheta),'*k','MarkerSize',18, 'LineWidth',2);
            text(itheta,pk_old(itheta)*1.2, ['Outgoing \gamma = ',num2str(itheta)],'FontSize',14);
        end
        set(gca,'FontSize',16, 'XLim',[1 N] );
        title('BPDN shrinkage constraints');
        %legend('Old constraint', 'New constraint', 'Element change','Location','NorthEast');
        %title(['BPDN constraints, n = ',num2str(N), ', m = ', num2str(M), ', T = ', num2str(T), ', Tin = ', num2str(T_in)]);
        plot(1:N, tau*ones(1,N),'--k');
        plot(1:N, -tau*ones(1,N), '--k');
        axis([1 N -tau*1.5 tau*1.5])
        
        figure(2);
        subplot(2,1,2)
        hold off
        plot(xp_old,'.r','MarkerSize',14); hold on;
        plot(xp_h,'LineWidth',1);
        set(gca,'FontSize',16,'XLim',[1 N]);
        title({['BPDN estimate at \epsilon = ',num2str(epsilon),', Iteration count = ',num2str(iter)]});
        if flag == 0
            plot(itheta, 0,'ok', 'MarkerSize',18,'LineWidth',2);
        else
            plot(itheta, 0,'or', 'MarkerSize',18,'LineWidth',2);
        end
        drawnow;
        pause(1/60);
    end
end
th = cputime-t0;