% Plot results of job_wtBPDN_xx 
%
% Written by: Salman Asif, Georgia Tech
% Email: sasif@ece.gatech.edu

clear
%% Setup path
mname = mfilename;
mpath = mfilename('fullpath');
mdir = mpath(1:end-length(mname));
cd(mdir);

addpath ..\utils_fig\
addpath ..\results_comparison_ALL\ 
rpath = '..\results_comparison_ALL\';

disp(['--------------------',datestr(now),'-------------------------']);

%% Simulation parameters
% reweighted setup
% rwt = 5;
% rwt_adp = 0; 

% simulation setup
maxsim = 100;

err_fun = @(z) -20*log10(z);
% Average over all the experiments using err_fun(mean(err^2)) (where err is
% the normalized error)
% In the submitted paper, I used the average of SER in db, which I later
% realized was probably not the best way to compute the average.
% To reproduce results in the paper, use 
% err_fun = @(z) -20*log10(z); 
% and compute mean(err_fun(err))

mType_list = {'randn','orth'};
sType_list = {'randn','sign','blocks','pcwPoly','HeaviSine'};
SNR_list = [30 40 inf];

SIM_list = {};
% Draft 1
% SIM_list{1} = 'comparison_wtBPDN_mT-randn_sT-randn_SNR30_rwt_mode5_lam_mode1';
% SIM_list{2} = 'comparison_wtBPDN_mT-randn_sT-randn_SNR40_rwt_mode5_lam_mode1';
% SIM_list{3} = 'comparison_wtBPDN_mT-randn_sT-randn_SNRinf_rwt_mode5_lam_mode1';
% SIM_list{4} = 'comparison_wtBPDN_mT-randn_sT-blocks_SNR30_rwt_mode5_lam_mode1';
% SIM_list{5} = 'comparison_wtBPDN_mT-randn_sT-blocks_SNR40_rwt_mode5_lam_mode1';
% SIM_list{6} = 'comparison_wtBPDN_mT-randn_sT-blocks_SNRinf_rwt_mode5_lam_mode1';
% SIM_list{7} = 'comparison_wtBPDN_mT-randn_sT-HeaviSine_SNR30_rwt_mode5_lam_mode1';
% SIM_list{8} = 'comparison_wtBPDN_mT-randn_sT-HeaviSine_SNR40_rwt_mode5_lam_mode1';
% SIM_list{9} = 'comparison_wtBPDN_mT-randn_sT-HeaviSine_SNRinf_rwt_mode5_lam_mode1';

% Reproduce randn('seed',2012), rand('seed',2012); 
% In the paper we have results for Blocks and HeaviSine at SNR = 40
SIM_list{1} = 'comparison_wtBPDN_mT-randn_sT-randn_SNR30-reproduce-Trwt';
SIM_list{2} = 'comparison_wtBPDN_mT-randn_sT-randn_SNR40-reproduce-Trwt';
SIM_list{3} = 'comparison_wtBPDN_mT-randn_sT-randn_SNRinf-reproduce-Trwt';
SIM_list{4} = 'comparison_wtBPDN_mT-randn_sT-blocks_SNR30-reproduce-Trwt';
SIM_list{5} = 'comparison_wtBPDN_mT-randn_sT-blocks_SNR40-reproduce-Trwt';
SIM_list{6} = 'comparison_wtBPDN_mT-randn_sT-blocks_SNRinf-reproduce-Trwt';
SIM_list{7} = 'comparison_wtBPDN_mT-randn_sT-HeaviSine_SNR30-reproduce-Trwt';
SIM_list{8} = 'comparison_wtBPDN_mT-randn_sT-HeaviSine_SNR40-reproduce-Trwt';
SIM_list{9} = 'comparison_wtBPDN_mT-randn_sT-HeaviSine_SNRinf-reproduce-Trwt';

%% Plot parameters
SAVE_results = true;
ppf = 3; % plots per figure 1 or 3?

exp1_mode = 1; % 0 - adp+irw; 1 - adp only

fsize = 18; % laptop 14, widescreen 18
MarkerSize = 10; % laptop 8, widescreen 10

N_plot = [256 512 1024];
Tr_plot = [3:5];

for sl = 4:length(SIM_list)
    eval(sprintf('load %s%s EXP_stack;',rpath,SIM_list{sl}));
    N_old = 256;
    
    mType = EXP_stack{2,1};
    sType = EXP_stack{2,2};
    SNR = EXP_stack{2,3};
    
    
    EXP = {'arw-h','rwt','yall1','sparsa','spgl1'};
    EXP = {'ARW-H','IRW-H','YALL1','SpaRSA','SPGL1','ORACLE','OMP'};
    MARKER = {'b*','rd','ko','ms','g^','bp','rv'};
    EXP_plot = [1:5];
    
    if sl > 3
        XTick = 0; XTickLabel = {'N,M'}; xtick = 0;
    else
        XTick = 0; XTickLabel = {'N,M,T'}; xtick = 0;
    end
    width = 325;
    fig1 = figure(1000); clf
    % set(gcf, 'Position', get(0,'Screensize')); % Maximize figure.
    set(gcf,'Position',[0 0 3*width+100 2*width+100]);

    fig2 = figure(2000); clf
    % set(gcf, 'Position', get(0,'Screensize')); % Maximize figure.
    set(gcf,'Position',[0 0 3*width+150 3*width+100]);

    fig3 = figure(3000); clf
    % set(gcf, 'Position', get(0,'Screensize')); % Maximize figure.       
    set(gcf,'Position',[0 0 3*width+150 3*width+100]);

    %     fig4 = figure(4000); clf
    %     % set(gcf, 'Position', get(0,'Screensize')); % Maximize figure.
    %     set(gcf,'Position',[0 0 3*width 2*width]);
    
    sim = 1;
    
    while sim < size(EXP_stack,1)
        sim = sim+1;
        N = EXP_stack{sim,6};
        M = EXP_stack{sim,7};
        T = EXP_stack{sim,8};
        SIM_memory = EXP_stack{sim,end};
        
        
        col = log2(N/256);
        if ppf == 1
            col = 0;
        end
        Tr = round(M/T);
        if (nnz(N == N_plot) && nnz(Tr == Tr_plot)) || sl > 3
            
            xtick = xtick + 1;
            XTick = [XTick; xtick];
            
            min_time = inf; max_time = 0;
            
            if sl > 2
                XTickLabel{xtick+1} = sprintf('%d,%d',N,M);
            else
                XTickLabel{xtick+1} = sprintf('%d,%d,%d',N,M,T);
            end
            
            % xticks = find(N==[EXP_temp1{:,4}]);
            % EXP_temp = [EXP_temp1(1,:); EXP_temp1(xticks,:)];
            
            % set(gcf, 'Position', get(0,'Screensize')); % Maximize figure.
            AXIS_cell = {};
            AXIS_cell{1,1} = 'SpacingVert'; AXIS_cell{2,1} = 0.01;
            AXIS_cell{1,2} = 'MT';          AXIS_cell{2,2} = 0.05;
            AXIS_cell{1,3} = 'MB';          AXIS_cell{2,3} = 0.125;
            AXIS_cell{1,4} = 'ML';          AXIS_cell{2,4} = 0.125;
            AXIS_cell{1,5} = 'MR';          AXIS_cell{2,5} = 0.025;
            AXIS_cell{1,6} = 'SpacingHoriz'; AXIS_cell{2,6} = 0.05;
            
            for exp = EXP_plot
                sim_err = [];
                sim_itr = [];
                sim_time = [];
                sim_supp_diff = [];
                for ii = 1:maxsim
                    sim_err = [sim_err; [SIM_memory{ii}{exp,3}]];
                    sim_itr = [sim_itr; [SIM_memory{ii}{exp,2}]];
                    sim_time = [sim_time; [SIM_memory{ii}{exp,5}]];
                    sim_supp_diff = [sim_supp_diff; [SIM_memory{ii}{exp,4}]];
                end
                
                %% SNR
                tmp_array = err_fun(sim_err);
                tmp_array1 = tmp_array(:,1);
                tmp_array2 = tmp_array(:,end);
                if exp == exp1_mode
                    tmp_array2 = tmp_array(:,1);
                end
                
                
                % subplot(411);
                figure(fig1)
                subaxis(2,ppf,1+col,AXIS_cell{:});
                % hdl = errorbar(xtick, mean(tmp_array1),std(tmp_array1),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                hdl = plot(xtick, (mean(tmp_array1)),MARKER{exp},'LineWidth',2,'MarkerSize',MarkerSize); hold on;
                axis tight; % grid minor;
                XLim = [1-(xtick-1)*0.1-eps xtick+(xtick-1)*0.1+eps];
                XTick(1) = XLim(1)/2;
                
                % axis properties similar for all plots...
                axis_prop = {};
                axis_prop{1,1} = 'XTick'; axis_prop{2,1} = XTick;
                axis_prop{1,2} = 'XTickLabel'; axis_prop{2,2} = [];
                axis_prop{1,3} = 'TickLength'; axis_prop{2,3} = [0.005;0.005];
                axis_prop{1,4} = 'FontSize'; axis_prop{2,4} = fsize;
                axis_prop{1,5} = 'XLim'; axis_prop{2,5} = XLim;
                axis_prop{1,6} = 'YGrid'; axis_prop{2,6} = 'on';
                axis_prop{1,7} = 'GridLineStyle'; axis_prop{2,7} = '--';
                set(gca, axis_prop{:});
                YLimF = get(gca,'YLim');
                if col == 0;
                    ylabel('Initial SER in dB','FontSize',fsize+2)
                end
                
                subaxis(2,ppf,ppf+1+col,'SpacingVert',0.01);
                % hdl = errorbar(xtick, mean(tmp_array2),std(tmp_array2),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                hdl = plot(xtick, (mean(tmp_array2)),MARKER{exp},'LineWidth',2,'MarkerSize',MarkerSize); hold on;
                axis tight;
                set(gca, axis_prop{:});
                set(gca, 'XTickLabel',XTickLabel);
                if col == 0
                    ylabel('Final SER in dB','FontSize',fsize+2)
                end
                
                YLimI = get(gca,'YLim');
                YLim = [min(YLimF(1),YLimI(1)) max(YLimF(2),YLimI(2))];
                YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                set(gca,'YLim',YLim);
                
                subaxis(2,ppf,1+col); set(gca,'YLim',YLim);
                
                % subaxis(2,1,2)
                % hText = xticklabel_rotate(XTick,30,XTickLabel,'FontSize',fsize);
                
                %% ITER
                tmp_array = sim_itr;
                tmp_array1 = tmp_array(:,1);
                tmp_array2 = sum(tmp_array(:,2:end),2);
                tmp_array3 = sum(tmp_array(:,1:end),2);
                if exp == exp1_mode
                    tmp_array2 = 0*tmp_array2;
                    tmp_array3 = tmp_array(:,1);
                end
                
                figure(fig2)
                subaxis(3,ppf,1+col,AXIS_cell{:});
                % hdl = errorbar(xtick, mean(tmp_array1),std(tmp_array1),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                hdl = plot(xtick, mean(tmp_array1),MARKER{exp},'LineWidth',2,'MarkerSize',MarkerSize); hold on;
                axis tight; % grid minor;
                set(gca, axis_prop{:});
                YLim = get(gca,'YLim');
                YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                set(gca,'YLim',YLim);
                if col == 0
                    ylabel('Initialization','FontSize',fsize+2);
                end
                
                if exp > exp1_mode
                    subaxis(3,ppf,ppf+1+col,AXIS_cell{:});
                    % hdl = errorbar(xtick, mean(tmp_array2),std(tmp_array2),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                    hdl = plot(xtick, mean(tmp_array2),MARKER{exp},'LineWidth',2,'MarkerSize',MarkerSize); hold on;
                    axis tight;
                    set(gca, axis_prop{:});
                    YLim = get(gca,'YLim');
                    YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                    YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                    set(gca,'YLim',YLim);
                    if col == 0
                        ylabel('Iterative reweighting','FontSize',fsize+2);
                    end
                end
                subaxis(3,ppf,2*ppf+1+col,AXIS_cell{:});
                % hdl = plot(xtick, mean(tmp_array3),std(tmp_array3),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                hdl = plot(xtick, mean(tmp_array3),MARKER{exp},'LineWidth',2,'MarkerSize',MarkerSize); hold on;
                axis tight;
                set(gca, axis_prop{:});
                set(gca, 'XTickLabel',XTickLabel);
                YLim = get(gca,'YLim');
                YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                set(gca,'YLim',YLim);
                if col == 0
                    ylabel('Total AtA count','FontSize',fsize+2);
                end
                
                %% TIME
                tmp_array = sim_time;
                tmp_array1 = tmp_array(:,1);
                tmp_array2 = sum(tmp_array(:,2:end),2);
                tmp_array3 = sum(tmp_array(:,1:end),2);
                if exp == exp1_mode
                    tmp_array2 = 0*tmp_array2;
                    tmp_array3 = tmp_array(:,1);
                end
                
                figure(fig3);
                subaxis(3,ppf,1+col,AXIS_cell{:});
                % hdl = errorbar(xtick, mean(tmp_array1),std(tmp_array1),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                hdl = plot(xtick, mean(tmp_array1),MARKER{exp},'LineWidth',2,'MarkerSize',MarkerSize); hold on;
                axis tight; % grid minor;
                set(gca, axis_prop{:});
                YLim = get(gca,'YLim');
                YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                set(gca,'YLim',YLim);
                if col == 0
                    ylabel('Initialization','FontSize',fsize+2);
                end
                
                if exp > exp1_mode
                    subaxis(3,ppf,ppf+1+col,'SpacingVert',0.01);
                    % hdl = errorbar(xtick, mean(tmp_array2),std(tmp_array2),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                    hdl = plot(xtick, mean(tmp_array2),MARKER{exp},'LineWidth',2,'MarkerSize',MarkerSize); hold on;
                    axis tight;
                    set(gca, axis_prop{:});
                    YLim = get(gca,'YLim');
                    YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                    YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                    set(gca,'YLim',YLim);
                    if col == 0
                        ylabel('Iterative reweighting','FontSize',fsize+2);
                    end
                end
                
                subaxis(3,ppf,2*ppf+1+col,'SpacingVert',0.01);
                % hdl = errorbar(xtick, mean(tmp_array3),std(tmp_array3),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                hdl = plot(xtick, mean(tmp_array3),MARKER{exp},'LineWidth',2,'MarkerSize',MarkerSize); hold on;
                axis tight;
                set(gca, axis_prop{:});
                set(gca, 'XTickLabel',XTickLabel);
                YLim = get(gca,'YLim');
                YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                set(gca,'YLim',YLim);
                if col == 0
                    ylabel({'Total runtime (sec.)'},'FontSize',fsize+2);
                end
                
                %% SUPP diff
                %                 tmp_array = sim_supp_diff;
                %                 tmp_array1 = tmp_array(:,1);
                %                 tmp_array2 = tmp_array(:,end);
                %
                %                 figure(fig4)
                %                 subaxis(2,ppf,1+col,AXIS_cell{:});
                %                 % hdl = errorbar(xtick, mean(tmp_array1),std(tmp_array1),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                %                 hdl = plot(xtick, mean(tmp_array1),MARKER{exp},'LineWidth',1.5,'MarkerSize',MarkerSize); hold on;
                %                 axis tight; % grid minor;
                %                 set(gca, axis_prop{:});
                %                 YLim = get(gca,'YLim');
                %                 YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                %                 YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                %                 set(gca,'YLim',YLim);
                %                 if col == 0
                %                     ylabel({'Initial supp. diff.'});
                %                 end
                %
                %
                %                 subaxis(2,ppf,ppf+1+col,'SpacingVert',0.01);
                %                 % hdl = plot(xtick, mean(tmp_array3),std(tmp_array3),MARKER{exp},'LineWidth',1.5,'MarkerSize',10); hold on;
                %                 hdl = plot(xtick, mean(tmp_array2),MARKER{exp},'LineWidth',1.5,'MarkerSize',MarkerSize); hold on;
                %                 axis tight;
                %                 set(gca, axis_prop{:});
                %                 set(gca, 'XTickLabel',XTickLabel);
                %                 YLim = get(gca,'YLim');
                %                 YLim(1) = YLim(1)-(YLim(2)-YLim(1))*0.1;
                %                 YLim(2) = YLim(2)+(YLim(2)-YLim(1))*0.1;
                %                 set(gca,'YLim',YLim);
                %                 if col == 0
                %                     ylabel({'Final supp. diff.'});
                %                 end
               
            end
        end
        if sim+1 <= size(EXP_stack,1)
            N = EXP_stack{sim+1,6};
        end
        if (N ~= N_old) || sim+1 > size(EXP_stack,1)
            for fig = 1:3
                eval(sprintf('figure(fig%d);',fig));
                if fig == 1 || fig == 4; ax = 2;
                else ax = 3; end
                subaxis(ax,ppf,(ax-1)*ppf+1+col);
                hText = xticklabel_rotate(XTick,45,XTickLabel,'FontSize',fsize);
                
                subaxis(ax,ppf,1+col,AXIS_cell{:});
                leg1 = legend(EXP{EXP_plot});
                set(leg1,'Orientation','Horizontal')
                % set(leg1,'Position',[0.075 0.96 fsize/150*length(EXP_plot) 0.04],'FontSize',fsize);
                set(leg1,'Position',[0.125 0.96 0.80 0.04],'FontSize',fsize);
                subaxis(ax,ppf,1+col);
                
                % msg = sprintf('mType-%s, sType-%s, SNR=%d: \n', mType, sType,SNR);
                msg = sprintf('%s-%s @ %d dB', mType, sType,SNR);
                % p=mtit(msg, 'fontsize',fsize,'color',[1 1 1]*.25, 'xoff',0,'yoff',0);                
                % set(p.th, 'Position',[0.9  0.92 0])
                if col == 2
                    subaxis(ax,ppf,1+col); shg;
                    title(msg); shg
                    % ylim=get(gca,'YLim');
                    % xlim=get(gca,'XLim');
                    % text(xlim(1),ylim(2),msg,...
                    %    'VerticalAlignment','bottom',...
                    %    'HorizontalAlignment','left', 'FontSize',fsize); shg
                end
            end
            
            N_old = N;
            XTick = 0; xtick = 0;
            % XTickLabel = {''};
            XTickLabel = XTickLabel(1);
            if ppf == 1
                if SAVE_results
                    saveppt2(pptname,'figure',fig1,'visible','notes',sprintf('%s rwtlam = %d \n error',msg,rwtlam),'halign','left','valign','top','stretch',false,'scale',true,'res',300)
                    saveppt2(pptname,'figure',fig2,'visible','notes',sprintf('%s rwtlam = %d \n iter',msg,rwtlam),'halign','left','valign','top','stretch',false,'scale',true,'res',300)
                    saveppt2(pptname,'figure',fig3,'visible','notes',sprintf('%s rwtlam = %d \n time',msg,rwtlam),'halign','left','valign','top','stretch',false,'scale',true,'res',300)
                end
                
                fig1 = figure(1000); clf
                fig2 = figure(2000); clf
                fig3 = figure(3000); clf                
            end
        end
    end
    %%
    if SAVE_results && ppf == 3
        % saveppt2(pptname,'figure',fig1,'visible','notes',sprintf('%s rwtlam = %d \n error',msg,rwtlam),'halign','left','valign','top','stretch',false,'scale',true,'res',300)
        % saveppt2(pptname,'figure',fig2,'visible','notes',sprintf('%s rwtlam = %d \n iter',msg,rwtlam),'halign','left','valign','top','stretch',false,'scale',true,'res',300)
        % saveppt2(pptname,'figure',fig3,'visible','notes',sprintf('%s rwtlam = %d \n time',msg,rwtlam),'halign','left','valign','top','stretch',false,'scale',true,'res',300)
        figure(fig1); set(gcf, 'Color', 'w');
        export_fig results_reproduce_Trwt.pdf -append
        % eval(sprintf('export_fig new2_results_publish_%s.pdf -append',rwtlam_extension));
        figure(fig2); set(gcf, 'Color', 'w');
        export_fig results_reproduce_Trwt.pdf -append
        % eval(sprintf('export_fig new2_results_publish_%s.pdf -append',rwtlam_extension));
        figure(fig3); set(gcf, 'Color', 'w');
        export_fig results_reproduce_Trwt.pdf -append
        % eval(sprintf('export_fig new2_results_publish_%s.pdf -append',rwtlam_extension));
    end
    
    pause(1);
end
