function [x varargout] = genSignal(N,in)

t = (1:N)./N;
if ~isfield(in,'randgen')
    randgen = 0;
else
    randgen = in.randgen;
end
wave_struct = [];
switch in.type
    case 'randn'
        T = in.T;
        q = randperm(N);
        x = zeros(N,1);
        x(q(1:T)) = randn(T,1);
    case 'sign'
        T = in.T;
        q = randperm(N);
        x = zeros(N,1);
        x(q(1:T)) = sign(randn(T,1));
    case 'ones'
        T = in.T;
        q = randperm(N);
        x = zeros(N,1);
        x(q(1:T)) = ones(T,1);
    case 'highD'
        T = in.T;
        q = randperm(N);
        x = zeros(N,1);
        x(q(1:ceil(T/2))) = sign(randn(ceil(T/2),1));
        x(q(ceil(T/2)+1:T)) = sign(randn(T-ceil(T/2),1))./1e2;
        
    case 'blocks'
        if randgen == 0
            pos = [ .1 .13 .15 .23 .25 .40 .44 .65  .76 .78 .81];
            hgt = [4 (-5) 3 (-4) 5 (-4.2) 2.1 4.3  (-3.1) 2.1 (-4.2)];
        else
            pos = sort(rand(1,10));
            % hgt = sign(randn(1,10)).*randint(1,10,[1 5]);
            hgt = sign(randn(1,10)).*randi([1 5],1,10);
        end
        sig = zeros(size(t));
        for j=1:length(pos)
            sig = sig + (1 + sign(t-pos(j))).*(hgt(j)/2) ;
        end
        sig = sig(:);
        
        % [h0 h1 g0 g1] = daub1018; sym = 2;
        % [h0 h1 g0 g1] = daub79; sym = 1;
        [h0 h1 g0 g1] = dauborth(2); sym = 0;
        
        J = floor(log2(N));
        x = fwt(sig,h0,h1,J,sym);
        
        sig_type = 'Blocks';
        wave_type = 'Haar wavelet';
        
    case 'pcwPoly'
        n = N;
        if randgen == 0
            t = (1:fix(n/5)) ./fix(n/5);
            sig1=20*(t.^3+t.^2+4);
            sig3=40*(2.*t.^3+t) + 100;
            sig2=10.*t.^3 + 45;
            sig4=16*t.^2+8.*t+16;
            sig5=20*(t+4);
            sig6(1:fix(n/10))=ones(1,fix(n/10));
            sig6=sig6*20;
            sig(1:fix(n/5))=sig1;
            sig(2*fix(n/5):-1:(fix(n/5)+1))=sig2;
            sig((2*fix(n/5)+1):3*fix(n/5))=sig3;
            sig((3*fix(n/5)+1):4*fix(n/5))=sig4;
            sig((4*fix(n/5)+1):5*fix(n/5))=sig5(fix(n/5):-1:1);
            diff=n-5*fix(n/5);
            sig(5*fix(n/5)+1:n)=sig(diff:-1:1);
            %sig((fix(n/20)+1):(fix(n/20)+fix(n/10)))=-ones(1,fix(n/10))*20;
            sig((fix(n/20)+1):(fix(n/20)+fix(n/10)))=ones(1,fix(n/10))*10;
            sig((n-fix(n/10)+1):(n+fix(n/20)-fix(n/10)))=ones(1,fix(n/20))*150;
            % zero-mean
            bias=sum(sig)/n;
            sig=sig-bias;
        else
            t = (1:fix(n/5)) ./fix(n/5);
            sig1=20*(t.^3+t.^2+4)*(1+(randn*.1));
            sig3=40*(2.*t.^3+t)*(1+(randn*.1)) + 100*(1+(randn*.05));
            sig2=10.*t.^3*(1+(randn*.05)) + 45*(1+(randn*.1));
            sig4=16*t.^2+8.*t*(1+randn*.1)+16*(1+(randn*.05));
            sig5=20*(t+4)*(1+(randn*.1));
            sig6(1:fix(n/10))=ones(1,fix(n/10))*(1+(randn*.1));
            sig6=sig6*20*(1+(randn*.05));
            sig(1:fix(n/5))=sig1;
            sig(2*fix(n/5):-1:(fix(n/5)+1))=sig2;
            sig((2*fix(n/5)+1):3*fix(n/5))=sig3;
            sig((3*fix(n/5)+1):4*fix(n/5))=sig4;
            sig((4*fix(n/5)+1):5*fix(n/5))=sig5(fix(n/5):-1:1);
            diff=n-5*fix(n/5);
            sig(5*fix(n/5)+1:n)=sig(diff:-1:1);
            %sig((fix(n/20)+1):(fix(n/20)+fix(n/10)))=-ones(1,fix(n/10))*20;
            sig((fix(n/20)+1):(fix(n/20)+fix(n/10)))=ones(1,fix(n/10))*10;
            sig((n-fix(n/10)+1):(n+fix(n/20)-fix(n/10)))=ones(1,fix(n/20))*150;
            % zero-mean
            bias=sum(sig)/n;
            sig=sig-bias;
        end
        sig = sig(:);
        
        [h0 h1 g0 g1] = dauborth(8); sym = 0;
        J = floor(log2(N));
        x = fwt(sig,h0,h1,J,sym);
        
        sig_type = 'Piecewise polynomial';
        wave_type = 'Daubechies-8 wavelet';
        
    case {'HeaviSine'}
        if randgen == 0
            sig = 4.*sin(4*pi.*t);
            sig = sig - sign(t - .3) - sign(.72 - t);
        else
            sig = (1+rand*.5)*4.*sin(4*pi.*t*(1+rand*.25));
            sig = sig - sign(t - .3*(1+randn*.1)) - sign(.72*(1+randn*.1) - t)*(1+randn);
        end
        sig = sig(:);
        
        [h0 h1 g0 g1] = dauborth(4); sym = 0;
        J = floor(log2(N));
        x = fwt(sig,h0,h1,J,sym);
        
        sig_type = 'HeaviSine';
        wave_type = 'Daubechies-4 wavelet';
        
    case {'Doppler'}
        if randgen == 0
            sig = sqrt(t.*(1-t)).*sin((2*pi*1.05) ./(t+.05));
        else
            sig = (1+randn*.02)*sqrt(t.*(1-t)).*sin((2*pi*1.05*(1+randn*.02)) ./(t+.05*(1+randn*.02)));
        end
        sig = sig(:);
        
        [h0 h1 g0 g1] = dauborth(4); sym = 0;
        J = floor(log2(N));
        x = fwt(sig,h0,h1,J,sym);
        
        sig_type = 'Doppler';
        wave_type = 'Daubechies-4 wavelet';
    case {'Cusp'}
        if randgen == 0
            sig = sqrt(abs(t - .37));
        else
            sig = sqrt(abs(t - (1+0.1*rand)*.37));
        end
        
        sig = sig(:);
        
        [h0 h1 g0 g1] = dauborth(4); sym = 0;
        J = floor(log2(N));
        x = fwt(sig,h0,h1,J,sym);
        
        sig_type = 'HeaviSine';
        wave_type = 'Daubechies-4 wavelet';
        
    case {'Ramp'}
        if randgen == 0
            sig = t - (t >= .37);
        else
            sig = (1+randn*.05)*t - (t >= .37*(1+randn*.02));
        end
        sig = sig(:);
        
        [h0 h1 g0 g1] = dauborth(4); sym = 0;
        J = floor(log2(N));
        x = fwt(sig,h0,h1,J,sym);
        
        sig_type = 'Cusp';
        wave_type = 'Daubechies-4 wavelet';
        
    case {'cameraman','barbara','lena','peppers','airplane','baboon','sailboat','tiffany','boats','shapes','pirate','house'}
        n = sqrt(N);
        switch in.type
            case {'shapes','boats'}
                load(in.type);    
            case {'pirate','cameraman'}
                sig = double(imread(in.type,'tif'));
            otherwise
                sig = double(imread(in.type,'ras')); 
        end
        sig = imresize(sig,n/size(sig,1));
        [h0 h1 g0 g1] = daub79; sym = 1; wave_type = 'daub79';
%         [h0 h1 g0 g1] = dauborth(4); sym = 0; wave_type = 'daub4';
%         [h0 h1 g0 g1] = dauborth(8); sym = 0; wave_type = 'daub8';
        
        fprintf('Image: %s, size: %dx%d, wavelet: %s \n',in.type, n,n, wave_type);
        J = floor(log2(n)-4);
        x = fwt2(sig,h0,h1,J,sym);
        x = x(:);
        N = length(x);
        
        wave_struct = {};
        wave_struct.wType = wave_type;
        wave_struct.J = J;
        wave_struct.FilterBank = [h0; h1; g0; g1];
        wave_struct.sym = sym;
        
    case 'phantom'
        n = sqrt(N);
        sig = phantom('Modified Shepp-Logan',n); 
        [h0 h1 g0 g1] = dauborth(2); sym = 0; wave_type = 'haar';
        
        fprintf('Image: %s, size: %dx%d, wavelet: %s \n',in.type, n,n, wave_type);
        J = floor(log2(n)-3);
        x = fwt2(sig,h0,h1,J,sym);
        x = x(:);
        N = length(x);        
    otherwise
        disp('NAO');
        
end

if isfield(in,'plot')
    if in.plot
        figure(1);
        subplot(211); set(gca,'FontSize',16);
        plot(sig,'LineWidth',2); set(gca,'XLim',[1 N]);
        axis tight; YLim = get(gca,'YLim');
        dY = YLim(2)-YLim(1);
        YLim = [YLim(1)-dY/10 YLim(2)+dY/10];
        set(gca,'YLim',YLim);
        set(gca,'XtickLabel','')
        ylabel(sig_type);
        subplot(212); set(gca,'FontSize',16);
        plot(x,'LineWidth',2); set(gca,'XLim',[1 N]);
        axis tight; YLim = get(gca,'YLim');
        dY = YLim(2)-YLim(1);
        YLim = [YLim(1)-dY/10 YLim(2)+dY/10];
        set(gca,'YLim',YLim);
        set(gca,'XtickLabel','')
        ylabel(wave_type);
    end
end

% Blocks, Pcw-poly, Ramp, etc. adopted from WAVELAB 850 package
% Originally made by David L. Donoho.
% Function has been enhanced.
%
%  Part of Wavelab Version 850
%  Built Tue Jan  3 13:20:39 EST 2006
%  This is Copyrighted Material
%  For Copying permissions see COPYING.m

nout = max(nargout,1) - 1;
if nout > 0
    varargout{1} = sig;
end
if nout > 1
    varargout{2} = wave_struct;
end

